@@ -45,6 +45,7 @@
 #include <linux/device.h>
 #include <linux/firmware.h>
 
+#include <pcmcia/cs_types.h>
 #include <pcmcia/cs.h>
 #include <pcmcia/cistpl.h>
 #include <pcmcia/ciscode.h>
@@ -188,11 +189,7 @@ static void bt3c_write_wakeup(bt3c_info_t *info)
 		return;
 
 	do {
-#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,36))
-		register unsigned int iobase = info->p_dev->resource[0]->start;
-#else
 		register unsigned int iobase = info->p_dev->io.BasePort1;
-#endif
 		register struct sk_buff *skb;
 		register int len;
 
@@ -230,11 +227,7 @@ static void bt3c_receive(bt3c_info_t *info)
 		return;
 	}
 
-#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,36))
-	iobase = info->p_dev->resource[0]->start;
-#else
 	iobase = info->p_dev->io.BasePort1;
-#endif
 
 	avail = bt3c_read(iobase, 0x7006);
 	//printk("bt3c_cs: receiving %d bytes\n", avail);
@@ -355,11 +348,7 @@ static irqreturn_t bt3c_interrupt(int irq, void *dev_inst)
 		/* our irq handler is shared */
 		return IRQ_NONE;
 
-#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,36))
-	iobase = info->p_dev->resource[0]->start;
-#else
 	iobase = info->p_dev->io.BasePort1;
-#endif
 
 	spin_lock(&(info->lock));
 
@@ -492,11 +481,7 @@ static int bt3c_load_firmware(bt3c_info_t *info, const unsigned char *firmware,
 	unsigned int iobase, size, addr, fcs, tmp;
 	int i, err = 0;
 
-#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,36))
-	iobase = info->p_dev->resource[0]->start;
-#else
 	iobase = info->p_dev->io.BasePort1;
-#endif
 
 	/* Reset */
 	bt3c_io_write(iobase, 0x8040, 0x0404);
@@ -673,13 +658,8 @@ static int bt3c_probe(struct pcmcia_device *link)
 	info->p_dev = link;
 	link->priv = info;
 
-#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,36))
-	link->resource[0]->flags |= IO_DATA_PATH_WIDTH_8;
-	link->resource[0]->end = 8;
-#else
 	link->io.Attributes1 = IO_DATA_PATH_WIDTH_8;
-	link->io.NumPorts1= 8;
-#endif
+	link->io.NumPorts1 = 8;
 
 #if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,35))
 	link->irq.Attributes = IRQ_TYPE_DYNAMIC_SHARING;
@@ -710,23 +690,14 @@ static int bt3c_check_config(struct pcmcia_device *p_dev,
 {
 	unsigned long try = (unsigned long) priv_data;
 
-#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,36))
-	p_dev->io_lines = (try == 0) ? 16 : cf->io.flags & CISTPL_IO_LINES_MASK;
-#endif
-
 	if (cf->vpp1.present & (1 << CISTPL_POWER_VNOM))
 		p_dev->conf.Vpp = cf->vpp1.param[CISTPL_POWER_VNOM] / 10000;
 	if ((cf->io.nwin > 0) && (cf->io.win[0].len == 8) &&
 	    (cf->io.win[0].base != 0)) {
-#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,36))
-		p_dev->resource[0]->start = cf->io.win[0].base;
-		if (!pcmcia_request_io(p_dev))
-#else
 		p_dev->io.BasePort1 = cf->io.win[0].base;
 		p_dev->io.IOAddrLines = (try == 0) ? 16 :
 			cf->io.flags & CISTPL_IO_LINES_MASK;
 		if (!pcmcia_request_io(p_dev, &p_dev->io))
-#endif
 			return 0;
 	}
 	return -ENODEV;
@@ -743,15 +714,9 @@ static int bt3c_check_config_notpicky(struct pcmcia_device *p_dev,
 
 	if ((cf->io.nwin > 0) && ((cf->io.flags & CISTPL_IO_LINES_MASK) <= 3)) {
 		for (j = 0; j < 5; j++) {
-#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,36))
-			p_dev->resource[0]->start = base[j];
-			p_dev->io_lines = base[j] ? 16 : 3;
-			if (!pcmcia_request_io(p_dev))
-#else
 			p_dev->io.BasePort1 = base[j];
 			p_dev->io.IOAddrLines = base[j] ? 16 : 3;
 			if (!pcmcia_request_io(p_dev, &p_dev->io))
-#endif
 				return 0;
 		}
 	}
