@@ -41,6 +41,7 @@
 #include <asm/system.h>
 #include <asm/io.h>
 
+#include <pcmcia/cs_types.h>
 #include <pcmcia/cs.h>
 #include <pcmcia/cistpl.h>
 #include <pcmcia/ciscode.h>
@@ -103,7 +104,7 @@ typedef struct {
 	u8 type;
 	u8 zero;
 	u16 len;
-} __packed nsh_t;	/* Nokia Specific Header */
+} __attribute__ ((packed)) nsh_t;	/* Nokia Specific Header */
 
 #define NSHL  4				/* Nokia Specific Header Length */
 
@@ -149,11 +150,7 @@ static void dtl1_write_wakeup(dtl1_info_t *info)
 	}
 
 	do {
-#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,36))
-		register unsigned int iobase = info->p_dev->resource[0]->start;
-#else
 		register unsigned int iobase = info->p_dev->io.BasePort1;
-#endif
 		register struct sk_buff *skb;
 		register int len;
 
@@ -218,11 +215,7 @@ static void dtl1_receive(dtl1_info_t *info)
 		return;
 	}
 
-#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,36))
-	iobase = info->p_dev->resource[0]->start;
-#else
 	iobase = info->p_dev->io.BasePort1;
-#endif
 
 	do {
 		info->hdev->stat.byte_rx++;
@@ -309,11 +302,7 @@ static irqreturn_t dtl1_interrupt(int irq, void *dev_inst)
 		/* our irq handler is shared */
 		return IRQ_NONE;
 
-#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,36))
-	iobase = info->p_dev->resource[0]->start;
-#else
 	iobase = info->p_dev->io.BasePort1;
-#endif
 
 	spin_lock(&(info->lock));
 
@@ -473,11 +462,7 @@ static int dtl1_hci_ioctl(struct hci_dev *hdev, unsigned int cmd,  unsigned long
 static int dtl1_open(dtl1_info_t *info)
 {
 	unsigned long flags;
-#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,36))
-	unsigned int iobase = info->p_dev->resource[0]->start;
-#else
 	unsigned int iobase = info->p_dev->io.BasePort1;
-#endif
 	struct hci_dev *hdev;
 
 	spin_lock_init(&(info->lock));
@@ -524,13 +509,7 @@ static int dtl1_open(dtl1_info_t *info)
 	outb(UART_LCR_WLEN8, iobase + UART_LCR);	/* Reset DLAB */
 	outb((UART_MCR_DTR | UART_MCR_RTS | UART_MCR_OUT2), iobase + UART_MCR);
 
-#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,36))
-	info->ri_latch = inb(info->p_dev->resource[0]->start + UART_MSR)
-				& UART_MSR_RI;
-#else
-	info->ri_latch = inb(info->p_dev->io.BasePort1 + UART_MSR)
-				& UART_MSR_RI;
-#endif
+	info->ri_latch = inb(info->p_dev->io.BasePort1 + UART_MSR) & UART_MSR_RI;
 
 	/* Turn on interrupts */
 	outb(UART_IER_RLSI | UART_IER_RDI | UART_IER_THRI, iobase + UART_IER);
@@ -555,11 +534,7 @@ static int dtl1_open(dtl1_info_t *info)
 static int dtl1_close(dtl1_info_t *info)
 {
 	unsigned long flags;
-#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,36))
-	unsigned int iobase = info->p_dev->resource[0]->start;
-#else
 	unsigned int iobase = info->p_dev->io.BasePort1;
-#endif
 	struct hci_dev *hdev = info->hdev;
 
 	if (!hdev)
@@ -597,15 +572,11 @@ static int dtl1_probe(struct pcmcia_device *link)
 	info->p_dev = link;
 	link->priv = info;
 
-#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,36))
-	link->resource[0]->flags |= IO_DATA_PATH_WIDTH_8;
-	link->resource[0]->end = 8;
-#else
 	link->io.Attributes1 = IO_DATA_PATH_WIDTH_8;
-	link->io.NumPorts1= 8;
-#endif
+	link->io.NumPorts1 = 8;
 #if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,35))
 	link->irq.Attributes = IRQ_TYPE_DYNAMIC_SHARING;
+
 	link->irq.Handler = dtl1_interrupt;
 #endif
 
@@ -631,20 +602,14 @@ static int dtl1_confcheck(struct pcmcia_device *p_dev,
 			  unsigned int vcc,
 			  void *priv_data)
 {
-	if ((cf->io.nwin != 1) || (cf->io.win[0].len <= 8))
-		return -ENODEV;
-
-#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,36))
-	p_dev->resource[0]->start = cf->io.win[0].base;
-	p_dev->resource[0]->end = cf->io.win[0].len;	/*yo */
-	p_dev->io_lines = cf->io.flags & CISTPL_IO_LINES_MASK;
-	return pcmcia_request_io(p_dev);
-#else
-	p_dev->io.BasePort1 = cf->io.win[0].base;
-	p_dev->io.NumPorts1 = cf->io.win[0].len;	/*yo */
-	p_dev->io.IOAddrLines = cf->io.flags & CISTPL_IO_LINES_MASK;
-	return pcmcia_request_io(p_dev, &p_dev->io);
-#endif
+	if ((cf->io.nwin == 1) && (cf->io.win[0].len > 8)) {
+		p_dev->io.BasePort1 = cf->io.win[0].base;
+		p_dev->io.NumPorts1 = cf->io.win[0].len;	/*yo */
+		p_dev->io.IOAddrLines = cf->io.flags & CISTPL_IO_LINES_MASK;
+		if (!pcmcia_request_io(p_dev, &p_dev->io))
+			return 0;
+	}
+	return -ENODEV;
 }
 
 static int dtl1_config(struct pcmcia_device *link)
@@ -653,11 +618,7 @@ static int dtl1_config(struct pcmcia_device *link)
 	int i;
 
 	/* Look for a generic full-sized window */
-#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,36))
-	link->resource[0]->end = 8;
-#else
 	link->io.NumPorts1 = 8;
-#endif
 	if (pcmcia_loop_config(link, dtl1_confcheck, NULL) < 0)
 		goto failed;
 
