@@ -114,10 +114,6 @@ ieee80211_bss_info_update(struct ieee80211_local *local,
 		bss->dtim_period = tim_ie->dtim_period;
 	}
 
-	/* If the beacon had no TIM IE, or it was invalid, use 1 */
-	if (beacon && !bss->dtim_period)
-		bss->dtim_period = 1;
-
 	/* replace old supported rates if we get new values */
 	srlen = 0;
 	if (elems->supp_rates) {
@@ -242,19 +238,20 @@ static bool ieee80211_prep_hw_scan(struct ieee80211_local *local)
 	local->hw_scan_req->n_channels = n_chans;
 
 	ielen = ieee80211_build_preq_ies(local, (u8 *)local->hw_scan_req->ie,
-					 req->ie, req->ie_len, band, (u32) -1,
-					 0);
+					 req->ie, req->ie_len, band);
 	local->hw_scan_req->ie_len = ielen;
 
 	return true;
 }
 
-static bool __ieee80211_scan_completed(struct ieee80211_hw *hw, bool aborted,
-				       bool was_hw_scan)
+void ieee80211_scan_completed(struct ieee80211_hw *hw, bool aborted)
 {
 	struct ieee80211_local *local = hw_to_local(hw);
+	bool was_hw_scan;
 
-	lockdep_assert_held(&local->mtx);
+	trace_api_scan_completed(local, aborted);
+
+	mutex_lock(&local->scan_mtx);
 
 	/*
 	 * It's ok to abort a not-yet-running scan (that
@@ -265,13 +262,17 @@ static bool __ieee80211_scan_completed(struct ieee80211_hw *hw, bool aborted,
 	if (WARN_ON(!local->scanning && !aborted))
 		aborted = true;
 
-	if (WARN_ON(!local->scan_req))
-		return false;
+	if (WARN_ON(!local->scan_req)) {
+		mutex_unlock(&local->scan_mtx);
+		return;
+	}
 
+	was_hw_scan = test_bit(SCAN_HW_SCANNING, &local->scanning);
 	if (was_hw_scan && !aborted && ieee80211_prep_hw_scan(local)) {
-		int rc = drv_hw_scan(local, local->scan_sdata, local->hw_scan_req);
-		if (rc == 0)
-			return false;
+		ieee80211_queue_delayed_work(&local->hw,
+					     &local->scan_work, 0);
+		mutex_unlock(&local->scan_mtx);
+		return;
 	}
 
 	kfree(local->hw_scan_req);
@@ -285,42 +286,26 @@ static bool __ieee80211_scan_completed(struct ieee80211_hw *hw, bool aborted,
 	local->scanning = 0;
 	local->scan_channel = NULL;
 
-	return true;
-}
-
-static void __ieee80211_scan_completed_finish(struct ieee80211_hw *hw,
-					      bool was_hw_scan)
-{
-	struct ieee80211_local *local = hw_to_local(hw);
+	/* we only have to protect scan_req and hw/sw scan */
+	mutex_unlock(&local->scan_mtx);
 
 	ieee80211_hw_config(local, IEEE80211_CONF_CHANGE_CHANNEL);
-	if (!was_hw_scan) {
-		ieee80211_configure_filter(local);
-		drv_sw_scan_complete(local);
-		ieee80211_offchannel_return(local, true);
-	}
+	if (was_hw_scan)
+		goto done;
 
-	mutex_lock(&local->mtx);
+	ieee80211_configure_filter(local);
+
+	drv_sw_scan_complete(local);
+
+	ieee80211_offchannel_return(local, true);
+
+ done:
 	ieee80211_recalc_idle(local);
-	mutex_unlock(&local->mtx);
-
 	ieee80211_mlme_notify_scan_completed(local);
 	ieee80211_ibss_notify_scan_completed(local);
 	ieee80211_mesh_notify_scan_completed(local);
 	ieee80211_queue_work(&local->hw, &local->work_work);
 }
-
-void ieee80211_scan_completed(struct ieee80211_hw *hw, bool aborted)
-{
-	struct ieee80211_local *local = hw_to_local(hw);
-
-	trace_api_scan_completed(local, aborted);
-
-	set_bit(SCAN_COMPLETED, &local->scanning);
-	if (aborted)
-		set_bit(SCAN_ABORTED, &local->scanning);
-	ieee80211_queue_delayed_work(&local->hw, &local->scan_work, 0);
-}
 EXPORT_SYMBOL(ieee80211_scan_completed);
 
 static int ieee80211_start_sw_scan(struct ieee80211_local *local)
@@ -364,8 +349,6 @@ static int __ieee80211_start_scan(struct ieee80211_sub_if_data *sdata,
 	struct ieee80211_local *local = sdata->local;
 	int rc;
 
-	lockdep_assert_held(&local->mtx);
-
 	if (local->scan_req)
 		return -EBUSY;
 
@@ -413,7 +396,19 @@ static int __ieee80211_start_scan(struct ieee80211_sub_if_data *sdata,
 	else
 		__set_bit(SCAN_SW_SCANNING, &local->scanning);
 
+	/*
+	 * Kicking off the scan need not be protected,
+	 * only the scan variable stuff, since now
+	 * local->scan_req is assigned and other callers
+	 * will abort their scan attempts.
+	 *
+	 * This avoids too many locking dependencies
+	 * so that the scan completed calls have more
+	 * locking freedom.
+	 */
+
 	ieee80211_recalc_idle(local);
+	mutex_unlock(&local->scan_mtx);
 
 	if (local->ops->hw_scan) {
 		WARN_ON(!ieee80211_prep_hw_scan(local));
@@ -421,6 +416,8 @@ static int __ieee80211_start_scan(struct ieee80211_sub_if_data *sdata,
 	} else
 		rc = ieee80211_start_sw_scan(local);
 
+	mutex_lock(&local->scan_mtx);
+
 	if (rc) {
 		kfree(local->hw_scan_req);
 		local->hw_scan_req = NULL;
@@ -447,8 +444,8 @@ ieee80211_scan_get_channel_time(struct ieee80211_channel *chan)
 	return IEEE80211_PROBE_DELAY + IEEE80211_CHANNEL_TIME;
 }
 
-static void ieee80211_scan_state_decision(struct ieee80211_local *local,
-					  unsigned long *next_delay)
+static int ieee80211_scan_state_decision(struct ieee80211_local *local,
+					 unsigned long *next_delay)
 {
 	bool associated = false;
 	bool tx_empty = true;
@@ -458,6 +455,12 @@ static void ieee80211_scan_state_decision(struct ieee80211_local *local,
 	struct ieee80211_sub_if_data *sdata;
 	struct ieee80211_channel *next_chan;
 
+	/* if no more bands/channels left, complete scan and advance to the idle state */
+	if (local->scan_channel_idx >= local->scan_req->n_channels) {
+		ieee80211_scan_completed(&local->hw, false);
+		return 1;
+	}
+
 	/*
 	 * check if at least one STA interface is associated,
 	 * check if at least one STA interface has pending tx frames
@@ -533,6 +536,7 @@ static void ieee80211_scan_state_decision(struct ieee80211_local *local,
 	}
 
 	*next_delay = 0;
+	return 0;
 }
 
 static void ieee80211_scan_state_leave_oper_channel(struct ieee80211_local *local,
@@ -648,17 +652,20 @@ void ieee80211_scan_work(struct work_struct *work)
 		container_of(work, struct ieee80211_local, scan_work.work);
 	struct ieee80211_sub_if_data *sdata = local->scan_sdata;
 	unsigned long next_delay = 0;
-	bool aborted, hw_scan, finish;
 
-	mutex_lock(&local->mtx);
-
-	if (test_and_clear_bit(SCAN_COMPLETED, &local->scanning)) {
-		aborted = test_and_clear_bit(SCAN_ABORTED, &local->scanning);
-		goto out_complete;
+	mutex_lock(&local->scan_mtx);
+	if (!sdata || !local->scan_req) {
+		mutex_unlock(&local->scan_mtx);
+		return;
 	}
 
-	if (!sdata || !local->scan_req)
-		goto out;
+	if (local->hw_scan_req) {
+		int rc = drv_hw_scan(local, sdata, local->hw_scan_req);
+		mutex_unlock(&local->scan_mtx);
+		if (rc)
+			ieee80211_scan_completed(&local->hw, true);
+		return;
+	}
 
 	if (local->scan_req && !local->scanning) {
 		struct cfg80211_scan_request *req = local->scan_req;
@@ -668,21 +675,21 @@ void ieee80211_scan_work(struct work_struct *work)
 		local->scan_sdata = NULL;
 
 		rc = __ieee80211_start_scan(sdata, req);
-		if (rc) {
-			/* need to complete scan in cfg80211 */
-			local->scan_req = req;
-			aborted = true;
-			goto out_complete;
-		} else
-			goto out;
+		mutex_unlock(&local->scan_mtx);
+
+		if (rc)
+			ieee80211_scan_completed(&local->hw, true);
+		return;
 	}
 
+	mutex_unlock(&local->scan_mtx);
+
 	/*
 	 * Avoid re-scheduling when the sdata is going away.
 	 */
 	if (!ieee80211_sdata_running(sdata)) {
-		aborted = true;
-		goto out_complete;
+		ieee80211_scan_completed(&local->hw, true);
+		return;
 	}
 
 	/*
@@ -692,12 +699,8 @@ void ieee80211_scan_work(struct work_struct *work)
 	do {
 		switch (local->next_scan_state) {
 		case SCAN_DECISION:
-			/* if no more bands/channels left, complete scan */
-			if (local->scan_channel_idx >= local->scan_req->n_channels) {
-				aborted = false;
-				goto out_complete;
-			}
-			ieee80211_scan_state_decision(local, &next_delay);
+			if (ieee80211_scan_state_decision(local, &next_delay))
+				return;
 			break;
 		case SCAN_SET_CHANNEL:
 			ieee80211_scan_state_set_channel(local, &next_delay);
@@ -715,19 +718,6 @@ void ieee80211_scan_work(struct work_struct *work)
 	} while (next_delay == 0);
 
 	ieee80211_queue_delayed_work(&local->hw, &local->scan_work, next_delay);
-	mutex_unlock(&local->mtx);
-	return;
-
-out_complete:
-	hw_scan = test_bit(SCAN_HW_SCANNING, &local->scanning);
-	finish = __ieee80211_scan_completed(&local->hw, aborted, hw_scan);
-	mutex_unlock(&local->mtx);
-	if (finish)
-		__ieee80211_scan_completed_finish(&local->hw, hw_scan);
-	return;
-
-out:
-	mutex_unlock(&local->mtx);
 }
 
 int ieee80211_request_scan(struct ieee80211_sub_if_data *sdata,
@@ -735,9 +725,9 @@ int ieee80211_request_scan(struct ieee80211_sub_if_data *sdata,
 {
 	int res;
 
-	mutex_lock(&sdata->local->mtx);
+	mutex_lock(&sdata->local->scan_mtx);
 	res = __ieee80211_start_scan(sdata, req);
-	mutex_unlock(&sdata->local->mtx);
+	mutex_unlock(&sdata->local->scan_mtx);
 
 	return res;
 }
@@ -748,9 +738,9 @@ int ieee80211_request_internal_scan(struct ieee80211_sub_if_data *sdata,
 {
 	struct ieee80211_local *local = sdata->local;
 	int ret = -EBUSY;
-	enum ieee80211_band band;
+	enum nl80211_band band;
 
-	mutex_lock(&local->mtx);
+	mutex_lock(&local->scan_mtx);
 
 	/* busy scanning */
 	if (local->scan_req)
@@ -785,44 +775,25 @@ int ieee80211_request_internal_scan(struct ieee80211_sub_if_data *sdata,
 
 	ret = __ieee80211_start_scan(sdata, sdata->local->int_scan_req);
  unlock:
-	mutex_unlock(&local->mtx);
+	mutex_unlock(&local->scan_mtx);
 	return ret;
 }
 
-/*
- * Only call this function when a scan can't be queued -- under RTNL.
- */
 void ieee80211_scan_cancel(struct ieee80211_local *local)
 {
 	bool abortscan;
-	bool finish = false;
+
+	cancel_delayed_work_sync(&local->scan_work);
 
 	/*
-	 * We are only canceling software scan, or deferred scan that was not
-	 * yet really started (see __ieee80211_start_scan ).
-	 *
-	 * Regarding hardware scan:
-	 * - we can not call  __ieee80211_scan_completed() as when
-	 *   SCAN_HW_SCANNING bit is set this function change
-	 *   local->hw_scan_req to operate on 5G band, what race with
-	 *   driver which can use local->hw_scan_req
-	 *
-	 * - we can not cancel scan_work since driver can schedule it
-	 *   by ieee80211_scan_completed(..., true) to finish scan
-	 *
-	 * Hence low lever driver is responsible for canceling HW scan.
+	 * Only call this function when a scan can't be
+	 * queued -- mostly at suspend under RTNL.
 	 */
+	mutex_lock(&local->scan_mtx);
+	abortscan = test_bit(SCAN_SW_SCANNING, &local->scanning) ||
+		    (!local->scanning && local->scan_req);
+	mutex_unlock(&local->scan_mtx);
 
-	mutex_lock(&local->mtx);
-	abortscan = local->scan_req && !test_bit(SCAN_HW_SCANNING, &local->scanning);
 	if (abortscan)
-		finish = __ieee80211_scan_completed(&local->hw, true, false);
-	mutex_unlock(&local->mtx);
-
-	if (abortscan) {
-		/* The scan is canceled, but stop work from being pending */
-		cancel_delayed_work_sync(&local->scan_work);
-	}
-	if (finish)
-		__ieee80211_scan_completed_finish(&local->hw, false);
+		ieee80211_scan_completed(&local->hw, true);
 }
